# Copyright (c) 2008-2019 by Enthought, Inc.
# All rights reserved.

"""
Version information for this Traits distribution.

This file is autogenerated by the Traits setup.py script.
"""

from __future__ import unicode_literals

#: The full version of the package, including a development suffix
#: for unreleased versions of the package.
version = "5.2.0"

#: The Git revision from which this release was made.
git_revision = "f28c2ae7e32d928bff46532bdd576b8ce74d6c51"
